@@ -220,11 +220,10 @@ impl Evaluator {
         });
 
         {
-            let data = self.data.clone();
             e.visit_mut_with(&mut pure_optimizer(
                 &serde_json::from_str("{}").unwrap(),
                 self.marks,
-                &data,
+                Eval::force_str_for_tpl(),
                 true,
                 false,
             ));
