@@ -8,7 +8,6 @@ use tracing::trace;
 
 use self::scope::Scope;
 use super::rename_map::RenameMap;
-use crate::pass::compute_char_freq::CharFreqInfo;
 
 mod scope;
 
@@ -19,16 +18,13 @@ pub(super) struct Analyzer {
 }
 
 impl Analyzer {
-    pub(super) fn into_rename_map(
-        mut self,
-        freq: &CharFreqInfo,
-        preserved: &FxHashSet<Id>,
-    ) -> AHashMap<Id, JsWord> {
+    pub(super) fn into_rename_map(mut self, preserved: &FxHashSet<Id>) -> AHashMap<Id, JsWord> {
         let mut map = RenameMap::default();
 
+        self.scope.prepare_renaming();
+
         let preserved_symbols = preserved.iter().cloned().map(|v| v.0).collect();
-        self.scope
-            .rename(freq, &mut map, preserved, &preserved_symbols);
+        self.scope.rename(&mut map, preserved, &preserved_symbols);
 
         map.into_map()
     }
